﻿using System;
using SDX.Compiler;
using Mono.Cecil;
using Mono.Cecil.Cil;
using System.Linq;
using System.Collections.Generic;

public class GameStagePartyFix : IPatcherMod
{
    public enum GameStagePartyPatchState
    {
        START,
        FINDINITPOS,
        FINDCALCPOS,
        FINDINSERTINCPOS,
        FINDINJUMP,
    }

    public bool Link(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        GameStagePartyPatch(gameModule);
        return true;
    }
    public void GameStagePartyPatch(ModuleDefinition gameModule)
    {
        TypeDefinition aiDirectorGameStagePartySpawner = gameModule.Types.First(d => d.Name == "AIDirectorGameStagePartySpawner");
        MethodDefinition resetPartyLevel = aiDirectorGameStagePartySpawner.Methods.First(d => d.Name == "ResetPartyLevel");

        TypeDefinition gameStageDefinition = gameModule.Types.First(d => d.Name == "GameStageDefinition");
        FieldDefinition diminishingReturns = gameStageDefinition.Fields.First(d => d.Name == "DiminishingReturns");
        FieldDefinition startingWeight = gameStageDefinition.Fields.First(d => d.Name == "StartingWeight");

        GameStagePartyPatchState state = GameStagePartyPatchState.START;
        Instruction initInsertIS=null;
        Instruction calcStartPosIS = null;
        Instruction incInsertPosIS = null;
        Instruction jumpLoopStartIS = null;
        foreach (Instruction inst in resetPartyLevel.Body.Instructions)
        {
            switch (state)
            {
                case GameStagePartyPatchState.START:
                    if (inst.OpCode == OpCodes.Stloc_S && inst.Operand.GetType() == typeof(VariableDefinition) && ((VariableDefinition)inst.Operand).Index == 5)
                    {
                        initInsertIS = inst.Next;
                        state = GameStagePartyPatchState.FINDINITPOS;
                    }
                    break;
                case GameStagePartyPatchState.FINDINITPOS:
                    if (inst.OpCode == OpCodes.Callvirt && inst.Operand.GetType() == typeof(MethodReference) && ((MethodReference)inst.Operand).FullName == "!0 System.Collections.Generic.List`1<System.Int32>::get_Item(System.Int32)")
                    {
                        calcStartPosIS = inst;
                        state = GameStagePartyPatchState.FINDCALCPOS;
                    }
                    break;
                case GameStagePartyPatchState.FINDCALCPOS:
                    if (inst.OpCode == OpCodes.Stloc_S && inst.Operand.GetType() == typeof(VariableDefinition) && ((VariableDefinition)inst.Operand).Index == 5)
                    {
                        incInsertPosIS = inst.Next;
                        state = GameStagePartyPatchState.FINDINSERTINCPOS;
                    }
                    break;
                case GameStagePartyPatchState.FINDINSERTINCPOS:
                    if (inst.OpCode == OpCodes.Blt_S)
                    {
                        jumpLoopStartIS = inst;
                        state = GameStagePartyPatchState.FINDINJUMP;
                    }
                    break;
            }


        }
        ILProcessor proc1 = resetPartyLevel.Body.GetILProcessor();
        VariableDefinition vd = new VariableDefinition("V_6", gameModule.Import(typeof(float)));
        proc1.Body.Variables.Add(vd);

        proc1.InsertBefore(initInsertIS, Instruction.Create(OpCodes.Ldsfld, startingWeight));
        proc1.InsertBefore(initInsertIS, Instruction.Create(OpCodes.Stloc_S, vd));

        proc1.Remove(calcStartPosIS.Next.Next);//inst IL_008c: ldc.r4 1 opcode ldc.r4 operand 1
        proc1.Remove(calcStartPosIS.Next.Next);//inst IL_0091: ldloc.s V_5 opcode ldloc.s operand V_5
        proc1.Remove(calcStartPosIS.Next.Next);//inst IL_0093: conv.r4 opcode conv.r4 operand null
        proc1.Remove(calcStartPosIS.Next.Next);//inst IL_0094: ldc.r4 0.2 opcode ldc.r4 operand 0.2
        proc1.Remove(calcStartPosIS.Next.Next);//inst IL_0099: mul opcode mul operand null
        proc1.Remove(calcStartPosIS.Next.Next);//inst IL_009a: sub opcode sub operand null
        Instruction insertPos = calcStartPosIS.Next.Next;

        Instruction jump2 = calcStartPosIS.Previous.Previous.Previous.Previous;

        proc1.InsertBefore(insertPos, Instruction.Create(OpCodes.Ldloc_S, vd));

        proc1.InsertBefore(incInsertPosIS, Instruction.Create(OpCodes.Ldloc_S, vd));
        proc1.InsertBefore(incInsertPosIS, Instruction.Create(OpCodes.Ldsfld, diminishingReturns));
        proc1.InsertBefore(incInsertPosIS, Instruction.Create(OpCodes.Sub));
        proc1.InsertBefore(incInsertPosIS, Instruction.Create(OpCodes.Stloc_S, vd));

        Instruction newJumpTo2 = Instruction.Create(OpCodes.Ldloc_S, vd);
        proc1.InsertBefore(incInsertPosIS, newJumpTo2);
        jump2.Operand = newJumpTo2;
        proc1.InsertBefore(incInsertPosIS, Instruction.Create(OpCodes.Ldc_R4, 0f));

        Instruction jump =  incInsertPosIS.Next.Next;
        proc1.Remove(incInsertPosIS.Next);//inst IL_00a7: ldc.i4.5 opcode ldc.i4.5 operand null
        proc1.Remove(incInsertPosIS);//inst IL_00a5: ldloc.s V_5 opcode ldloc.s operand V_5
        jump.OpCode = OpCodes.Ble;
    }
    public bool Patch(ModuleDefinition module)
    {
        Console.WriteLine("==GameStagePartyFix Patcher===");
        return true;
    }


}
